ERR_MODULE_GENERAL = "general"
ERR_MODULE_INTERNAL = "internal"
ERR_MODULE_IO = "io"
ERR_MODULE_MATCHING = "matching"
ERR_MODULE_SUPERVISION = "supervision"
ERR_MODULE_TEMPLATE = "template"

# TODO: remove the module system
