"""
Module containing the OfficialEye internal API.

WARNING!
Do not import it unless you know precisely what you are doing.
Instead, use the public API to interact with OfficialEye programatically.
"""

import z3

z3.set_param("parallel.enable", True)
