"""
This module contains everything that should be visible to the API user at the top level of imports,
i.e., all symbols that should be importable via `from officialeye import symbol`
"""

# TODO: write docstrings for all public methods of the api
