"""
This module provides a set of functions connecting the API interface with the internal implementation interface.
In other words, the functions of this module form a low-level API that should be called internally in the actual public API.
"""