"""
Module providing an API for all OfficialEye's document detection tools.
"""

# disable unused imports ruff check
# ruff: noqa: F401

# noinspection PyProtectedMember,PyUnresolvedReferences
from officialeye._api.detection import detect
